package com.example.lanes;

import android.app.Activity;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PointF;
import android.graphics.RectF;
import android.graphics.Typeface;
import android.os.Bundle;
import android.view.Menu;

import com.mindfusion.diagramming.AnchorPattern;
import com.mindfusion.diagramming.AnchorPoint;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramAdapter;
import com.mindfusion.diagramming.DiagramLink;
import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.DrawItemEvent;
import com.mindfusion.diagramming.HandlesStyle;
import com.mindfusion.diagramming.HitTestEvent;
import com.mindfusion.diagramming.LinkEvent;
import com.mindfusion.diagramming.LinkShape;
import com.mindfusion.diagramming.MarkStyle;
import com.mindfusion.diagramming.NodeAdjustmentHandle;
import com.mindfusion.diagramming.NodeEvent;
import com.mindfusion.diagramming.NodeHandleType;
import com.mindfusion.diagramming.NodeValidationEvent;
import com.mindfusion.diagramming.Orientation;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.ShapeNode;
import com.mindfusion.diagramming.SnapToAnchor;
import com.mindfusion.diagramming.Utilities;
import com.mindfusion.diagramming.lanes.Grid;
import com.mindfusion.diagramming.lanes.Header;
import com.mindfusion.diagramming.lanes.HeaderList;
import com.mindfusion.diagramming.lanes.ICell;
import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.Font;
import com.mindfusion.drawing.GraphicsUnit;
import com.mindfusion.drawing.Pen;
import com.mindfusion.drawing.SolidBrush;

import java.util.ArrayList;

public class MainActivity extends Activity
{
	private DiagramView diagramView;
	private Diagram diagram;
	
	static AnchorPattern pattern = new AnchorPattern(
		new AnchorPoint[] {
			new AnchorPoint(100, 50, false, true, MarkStyle.None),
			new AnchorPoint(0, 0, true, false, MarkStyle.None)
		}, "myPattern");

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		diagramView = (DiagramView)findViewById(R.id.diag_view);
		diagram = diagramView.getDiagram();

		initDiagram();
		if (savedInstanceState == null)
			createGrid();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
	
	private void initDiagram()
	{
		// initialize diagram
		diagram.setFont(new Font("Roboto", Typeface.NORMAL, 12));
		diagram.setMeasureUnit(GraphicsUnit.Point);
		diagram.setBackBrush(new SolidBrush(new Color(240,240,250)));
		diagram.setLinkShape(LinkShape.Cascading);
		diagram.setLinkSegments((short)2);
		diagram.setLinkCascadeOrientation(Orientation.Horizontal);
		diagram.setSnapToAnchor(SnapToAnchor.OnCreateOrModify);
		diagram.getDiagramLinkStyle().setBrush(new SolidBrush(Color.black));
		diagram.setLinkHeadShape(Shape.fromId("Alternative"));
		diagram.setLinkHeadShapeSize(1.4f);
		diagram.getShapeNodeStyle().setBrush(new SolidBrush(new Color(40, 0, 0, 0)));
		diagram.setShadowOffsetX(1f);
		diagram.setShadowOffsetY(1f);
		diagram.setDefaultShape(Shape.fromId("Rectangle"));
		diagram.setAllowSelfLoops(false);
		diagram.getShapeNodeStyle().setBrush(new SolidBrush(new Color(247,247,242)));
		diagram.setAlignToGrid(false);

		// handle diagram events
		diagram.addDiagramListener(new DiagramAdapter()
		{
			public void linkCreated(LinkEvent e) 
			{
				diagramLinkCreated(e);
			}
			public void nodeCreated(NodeEvent e) 
			{
				diagramNodeCreated(e);
			}
			public void nodeModified(NodeEvent e) 
			{
				diagramNodeModified(e);
			}
			public void nodeCreating(NodeValidationEvent e)
			{
				diagramNodeCreating(e);
			}
			public void nodeModifying(NodeValidationEvent e)
			{
				diagramNodeModifying(e);
			}

			// custom adjustment handles hit-testing;
			// allow left and right resize when pointer is near the node's edge
			// and move when near the center
			public void hitTestAdjustmentHandles(HitTestEvent e)
            {
                if (!(e.getItem() instanceof ShapeNode))
                {
                   e.setHitResult(null);
                   return;
                }

                ShapeNode box = (ShapeNode) e.getItem();
                PointF pos = e.getPointerPosition();
                if (!box.getBounds().contains(pos.x, pos.y))
                {
                   e.setHitResult(null);
                   return;
                }

                RectF left = box.getBounds();
                Utilities.setWidth(left, 10);

                if (left.contains(pos.x, pos.y))
                {
                   e.setHitResult(new NodeAdjustmentHandle(NodeHandleType.ResizeMiddleLeft));
                   return;
                }

                RectF right = box.getBounds();
                right.left = right.right - 10;

                if (right.contains(pos.x, pos.y))
                {
                   e.setHitResult(new NodeAdjustmentHandle(NodeHandleType.ResizeMiddleRight));
                   return;
                }

                RectF middle = box.getBounds();
                middle.offsetTo(middle.left + middle.width() / 4, middle.top);
                Utilities.setWidth(middle, middle.width() / 2);

                if (middle.contains(pos.x, pos.y))
                {
                   e.setHitResult(new NodeAdjustmentHandle(NodeHandleType.Move));
                   return;
                }

                e.setHitResult(null);
            }

			// custom adjustment handles drawing;
			// draw lines near left and right edges to represent resize grips
			public void drawAdjustmentHandles(DrawItemEvent e)
			{
				if (resizeStroke == null)
				{
					resizeStroke = new Paint();
					resizeStroke.setColor(Color.black.value());
					resizeStroke.setStyle(Paint.Style.STROKE);
				}
				Canvas canvas = e.getGraphics();
				DiagramNode node = (DiagramNode)e.getItem();
				RectF r = node.getBounds();
				canvas.drawLine(r.left + 3, r.top, r.left + 3, r.bottom, resizeStroke);
				canvas.drawLine(r.right - 3, r.top, r.right - 3, r.bottom, resizeStroke);
			}
			
			Paint resizeStroke;
		});
	}

	private void createGrid()
	{
		// initialize grid
		Header header;
		Grid grid = diagram.getLaneGrid();
		HeaderList columns = grid.getColumnHeaders();

		grid.setMinHeaderSize(25);
		grid.setHookHeaders(false);
		grid.setHeadersOnTop(false);
		grid.setColumnHeadersHeights(new float[] { 25, 25 });
		grid.setAllowResizeHeaders(true);

		// create week and day columns
		for (int i = 1; i <= 8; i++)
		{
			header = new Header("Week " + i + ", 2014");
			header.getSubHeaders().add(new Header("S"));
			header.getSubHeaders().add(new Header("M"));
			header.getSubHeaders().add(new Header("T"));
			header.getSubHeaders().add(new Header("W"));
			header.getSubHeaders().add(new Header("T"));
			header.getSubHeaders().add(new Header("F"));
			header.getSubHeaders().add(new Header("S"));
			columns.add(header);
		}

		// create task rows
		for (int i = 1; i <= 6; i++)
		{
			header = new Header("Task " + i);
			header.setWidth(65);
			grid.getRowHeaders().add(header);
		}

		// column styling
		int count = 8 * 7;
		for (int i = 0; i < count; i++)
		{
			Header column = grid.getColumn(i);

			// remove all borders except ones between weeks
			if (i % 7 != 0)
			{
				grid.get(column, null).getStyle().setLeftBorderPen(null);
			}
			else
			{
				grid.get(column, null).getStyle().setLeftBorderPen(new Pen(0, Color.gray));
			}

			if (i != count - 1)
				grid.get(column, null).getStyle().setRightBorderPen(null);

			// weekend fill color
			if (i % 7 == 0 ||
					i % 7 == 6)
			{
				grid.get(column, null).getStyle().setBackgroundBrush(
						new SolidBrush(new Color(238, 232, 170)));
			}
		}

		// row styling
		count = 6;
		for (int i = 0; i < count; i++)
		{
			Header row = grid.getRow(i);

			if (i == 0)
			{
				grid.get(null, row).getStyle().setTopBorderPen(null);
			}
			else
			{
				grid.get(null, row).getStyle().setTopBorderPen(new Pen(0, Color.gray));
			}

			if (i != count - 1)
				grid.get(null, row).getStyle().setBottomBorderPen(null);
		}

		// show the grid
		diagram.setEnableLanes(true);

		// ensure the document is big enough to contain the grid
		float width = Math.max(150, grid.getColumnCount() * 25 + 65);
		float height = Math.max(150, grid.getRowCount() * 25 + 60);
		diagram.setBounds(new RectF(0, 0, width, height));
	}
	
	/// <summary>
	/// Gets the size of a pixel measured in document units.
	/// </summary>
	private float getPixel()
	{
		PointF pt1 = diagramView.deviceToDoc(0, 0);
		PointF pt2 = diagramView.deviceToDoc(1, 1);
		
		return pt2.x - pt1.x;
	}

	/// <summary>
	/// Fixes the segments of the specified arrow according
	/// to the position of its related boxes.
	/// </summary>
	private void setLinkShape(DiagramLink a)
	{
		RectF originBounds = new RectF(a.getOrigin().getBounds());
		RectF destBounds = new RectF(a.getDestination().getBounds());

		if (originBounds.right < destBounds.left)
		{
			a.setSegmentCount(2);
		}
		else
		{
			a.setSegmentCount(4);

			PointF pt0 = a.getControlPoints().getCopy(0);
			PointF pt1;
			PointF pt2;
			PointF pt3 = a.getControlPoints().getCopy(3);
			PointF pt4 = a.getControlPoints().getCopy(4);

			pt1 = new PointF(pt0.x, pt0.y);
			pt1.x += 6;

			pt2 = new PointF(pt1.x, pt1.y);
			pt2.y += 12;

			pt3.x = pt4.x;
			pt3.y = pt2.y;

			a.getControlPoints().set(1, pt1);
			a.getControlPoints().set(2, pt2);
			a.getControlPoints().set(3, pt3);

			a.updateFromPoints();
		}
	}

	private void diagramNodeCreated(NodeEvent e)
	{
		e.getNode().setHandlesStyle(HandlesStyle.Custom);
		e.getNode().setAnchorPattern(pattern);

		// Place the box within the grid
		RectF bounds = new RectF(e.getNode().getBounds());
		PointF topLeft = new PointF(bounds.left, bounds.top);

		Grid grid = diagram.getLaneGrid();

		RectF cellBounds = new RectF();
		grid.getCellFromPoint(topLeft, cellBounds);

		float pixel = getPixel();

		bounds.offsetTo(bounds.left, cellBounds.top + 4 * pixel);
		Utilities.setHeight(bounds, cellBounds.height() - 10 * pixel);
		e.getNode().setBounds(bounds);
	}

	private void diagramNodeCreating(NodeValidationEvent e)
	{
		RectF bounds = new RectF(e.getNode().getBounds());
		PointF topLeft = new PointF(bounds.left, bounds.top);

		Grid grid = diagram.getLaneGrid();

		RectF cellBounds = new RectF();
		ICell cell = grid.getCellFromPoint(topLeft, cellBounds);

		if (cell == null)
			e.setCancel(true);
	}
	
	private void diagramNodeModifying(NodeValidationEvent e)
	{
		RectF bounds = new RectF(e.getNode().getBounds());
		PointF topLeft = new PointF(bounds.left, bounds.top + bounds.height() / 2);

		Grid grid = diagram.getLaneGrid();

		RectF cellBounds = new RectF();
		ICell cell = grid.getCellFromPoint(topLeft, cellBounds);

		if (cell == null)
			e.setCancel(true);
	}

	private void diagramLinkCreated(LinkEvent e)
	{
		e.getLink().setAllowMoveStart(false);
		e.getLink().setAllowMoveEnd(false);
		setLinkShape(e.getLink());
	}

	private void diagramNodeModified(NodeEvent e)
	{
		// Place the box within the grid
		RectF bounds = new RectF(e.getNode().getBounds());
		PointF topLeft = new PointF(bounds.left, bounds.top + bounds.height() / 2);

		Grid grid = diagram.getLaneGrid();

		RectF cellBounds = new RectF();
		grid.getCellFromPoint(topLeft, cellBounds);

		float pixel = getPixel();

		bounds.offsetTo(bounds.left, cellBounds.top + 4 * pixel);
		Utilities.setHeight(bounds, cellBounds.height() - 10 * pixel);
		e.getNode().setBounds(bounds);

		// Fix links
		ArrayList<DiagramLink> links = new ArrayList<DiagramLink>();
		links.addAll(e.getNode().getIncomingLinks());
		links.addAll(e.getNode().getOutgoingLinks());

		for (DiagramLink link : links)
		{
			setLinkShape(link);
		}	
	}
}