package com.example.layeredlayout;

import java.util.Random;

import com.mindfusion.diagramming.AnchorPattern;
import com.mindfusion.diagramming.Anchoring;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.LayeredLayout;
import com.mindfusion.diagramming.ShapeNode;

import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.GradientBrush;
import com.mindfusion.drawing.SolidBrush;

import android.os.Bundle;
import android.app.Activity;
import android.view.Menu;
import android.view.Surface;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;

public class MainActivity extends Activity
{
	private DiagramView diagramView;
	private Diagram diagram;
	private Random random = new Random();
	private boolean landscape;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		WindowManager wm = (WindowManager)getSystemService(WINDOW_SERVICE);
		int rotation = wm.getDefaultDisplay().getRotation();
		landscape = rotation == Surface.ROTATION_90 || rotation == Surface.ROTATION_270;
		
		diagramView = (DiagramView)findViewById(R.id.diag_view);
		diagram = diagramView.getDiagram();
		initDiagram();
		
		Button btnGenerate = (Button)findViewById(R.id.generate);
        btnGenerate.setOnClickListener(new View.OnClickListener()
        {
			public void onClick(View v)
			{
				arrangeDiagram();
			}
		});
	}

	private void initDiagram()
	{
		diagram.setBackBrush(new SolidBrush(Color.white));
		diagram.getShapeNodeStyle().setBrush(new GradientBrush(
				new Color(111, 127, 159), new Color(207, 223, 255), 30));	
	}
	
	private void arrangeDiagram()
	{
		randomGraph();
        arrange();
	}
	
	private void randomGraph()
	{
		diagram.clearAll();

        for (int i = 0; i < 20; ++i)
        {
            int c = diagram.getNodes().size();
            int g = 2 + random.nextInt(10);
            for (int j = 0; j < g; ++j)
            {
                ShapeNode node = diagram.getFactory().createShapeNode(0, 0, 40, 40);
                node.setAnchorPattern(landscape ?
                	AnchorPattern.fromId("TopInBottomOut") : AnchorPattern.fromId("LeftInRightOut"));
                if (j > 0)
                    diagram.getFactory().createDiagramLink(diagram.getNodes().get(diagram.getNodes().size() - 2), node);
            }
            if (i > 0)
            {
                for (int j = 0; j < 1 + random.nextInt(3); ++j)
                    diagram.getFactory().createDiagramLink(
                        diagram.getNodes().get(random.nextInt(c)),
                        diagram.getNodes().get(c + random.nextInt(g)));
            }
        }
	}
	
    private void arrange()
    {
    	LayeredLayout layout = new LayeredLayout();
        layout.setAnchoring(Anchoring.Reassign);
        layout.setEnforceLinkFlow(true);
        layout.setStraightenLongLinks(true);
        if (landscape)
        {
        	layout.setNodeDistance(15);
        	layout.setLayerDistance(10);
        	layout.setOrientation(com.mindfusion.diagramming.jlayout.Orientation.Vertical);
        }
        else
        {
        	layout.setNodeDistance(10);
        	layout.setLayerDistance(15);
        	layout.setOrientation(com.mindfusion.diagramming.jlayout.Orientation.Horizontal);
        }
        layout.arrange(diagram);

        diagram.resizeToFitItems(5);
        diagramView.zoomToFit(diagram.getBounds());
    }
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
}