package com.example.stockshapes;

import android.app.Activity;
import android.os.Bundle;
import android.view.Menu;

import com.mindfusion.diagramming.Align;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.Group;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.ShapeNode;
import com.mindfusion.diagramming.ShapeNodeStyle;
import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.GradientBrush;
import com.mindfusion.drawing.GraphicsUnit;
import com.mindfusion.drawing.SolidBrush;

import java.util.Arrays;


public class MainActivity extends Activity
{
	private DiagramView diagramView;
	private Diagram diagram;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		diagramView = (DiagramView)findViewById(R.id.diag_view);
		diagram = diagramView.getDiagram();

		initDiagram();
		if (savedInstanceState == null)
			createShapes();
	}

	private void initDiagram()
	{
		diagram.setSelectAfterCreate(false);
		
		diagram.setBackBrush(new SolidBrush(new Color(234,255,255)));
		diagram.getDiagramStyle().setShadowBrush(new SolidBrush(new Color(242,242,242)));
	
		ShapeNodeStyle shapesStyle = diagram.getShapeNodeStyle(); 
		shapesStyle.setBrush(new GradientBrush(new Color(255, 255, 255, 100),
			new Color(0, 128, 192, 255), 45));
		shapesStyle.setStroke(new SolidBrush(Color.black));
		shapesStyle.setFontFamily("Roboto");
		shapesStyle.setFontSize(3);
		shapesStyle.setFontUnit(GraphicsUnit.Millimeter);
	}

	private void createShapes()
	{
		int i = 0;
		int hsize = 18;
		int vsize = 18;
		int padding = 12;

		int perLine = 8;

		String[] ids = new String[Shape.getShapes().size()];
		int a = 0;
		for (Shape shape : Shape.getShapes())
			ids[a++] = shape.getId();
		Arrays.sort(ids);

		// enum all predefined shapes
		for (int j = 0; j < ids.length; j++)
		{
			Shape shape = Shape.fromId(ids[j]);

			// skip some arrowhead shapes that aren't that useful as node shapes
			if (shape.getOutline().length == 0)
				continue;
			if (shape.getId() == "RevWithCirc")
				continue;
			if (shape.getId() == "DoubleArrow")
				continue;

			// create a box of this shape
			ShapeNode node = diagram.getFactory().createShapeNode(
					(i % perLine) * (hsize + padding) + 2,
					(i / perLine) * (vsize + 15) + 4,
					hsize, vsize);
			node.setShape(shape);

			// attach text below the box
			Group labels = diagram.getFactory().createGroup(node);
			ShapeNode label = diagram.getFactory().createShapeNode(
					node.getCenter().x - hsize / 2 - 2,
					(i / perLine) * (vsize + 15) + vsize + 4,
					hsize + 16, 18);
			label.setShape(Shape.fromId("Rectangle"));
			label.getTextFormat().setVerticalAlign(Align.Near);
			label.setText(shape.getId().startsWith("Bpmn") ?
					"Bpmn\n\r" + shape.getId().substring(4) : shape.getId());
			label.setLocked(true);
			label.setTransparent(true);
			label.moveTo(
					(float)(node.getCenter().x - label.getBounds().width() / 2),
					(float)label.getBounds().top);

			labels.attachToCorner(label, 3);

			i = i + 1;
		}

		diagram.resizeToFitItems(4, true);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
}