package com.mindfusion.tutorial4;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;

import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramAdapter;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.NodeEvent;
import com.mindfusion.diagramming.NodeListView;
import com.mindfusion.drawing.Color;
import com.mindfusion.drawing.SolidBrush;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

public class MainActivity extends Activity
{
	DiagramView view;
	private Diagram diagram;
	private Button btnUndo;
	private Button btnRedo;
	private Button btnSave;
	private Button btnLoad;
	
	private Activity getThis(){
		return this;
	}
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		Diagram.registerClass(OrgChartNode.class, "my:OrgChartNode", 1);
		
		view  = (DiagramView)findViewById(R.id.diag_view);
		diagram = view.getDiagram();
		diagram.getUndoManager().setUndoEnabled(true);
		diagram.setBackBrush(new SolidBrush(Color.WHITE));

		// enable drawing OrgChartNodes using touch input
		view.setCustomBehavior(new OrgChartBehavior(view));
		
		btnSave = (Button)findViewById(R.id.btn_save);
		btnLoad = (Button)findViewById(R.id.btn_load);
		btnUndo = (Button)findViewById(R.id.btn_undo);
		btnRedo = (Button)findViewById(R.id.btn_redo);		

		// set up the NodeListView
		NodeListView nodeList = (NodeListView)findViewById(R.id.list_view);
		nodeList.setDefaultNodeSize(new com.mindfusion.drawing.Dimension(80, 40));
		nodeList.setIconSize(new com.mindfusion.drawing.Dimension(60, 30));
		nodeList.setIconMargin(new com.mindfusion.drawing.Dimension(5, 5));
		nodeList.setDiagramView(view);
		nodeList.setBackgroundColor(android.graphics.Color.argb(255, 200, 200, 225));

		// add nodes to list view
		OrgChartNode orgNode = new OrgChartNode(diagram, this);
		orgNode.setTitle("CEO");
		orgNode.setFullName("Type here");
		orgNode.setText("Chief executive officer");
		orgNode.setImage(loadImage("image1.png"));
		nodeList.addNode(orgNode);
		
		orgNode = new OrgChartNode(diagram, this);
		orgNode.setTitle("CIO");
		orgNode.setFullName("Type here");
		orgNode.setText("Chief information officer");
		orgNode.setImage(loadImage("image2.png"));
		nodeList.addNode(orgNode);
		
		btnSave.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				try {
					diagram.saveTo("save.bin", getThis());
				} catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		});
		
		btnLoad.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				try {
					diagram.loadFrom("save.bin", getThis());
				} catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
				
			}
		});
		
		btnUndo.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				diagram.getUndoManager().undo();	
			}
		});
		
		btnRedo.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View v) {
				diagram.getUndoManager().redo();
			}
		});
		
		diagram.addDiagramListener(new DiagramAdapter() {
			@Override
			public void nodeCreated(NodeEvent e)
			{
				OrgChartNode node = (OrgChartNode) e.getNode();
				if (node != null && node.getImage() == null)
				{
					node.setImage(loadImage("image1.png"));
					node.setTitle("new node");
					node.setFullName("new node");
					node.setText("new node");
				}
			}
		});		
	}

	Bitmap loadImage(String fileName)
	{
		Bitmap image = null;
		try
		{
			InputStream istream = getAssets().open(fileName);
			image = BitmapFactory.decodeStream(istream);
		}
		catch (IOException e1)
		{
			e1.printStackTrace();
		}
		return image;
	}

	@Override
	protected void onRestoreInstanceState(Bundle savedInstanceState)
	{
		super.onRestoreInstanceState(savedInstanceState);
		// enable drawing OrgChartNodes using touch input
		view.setCustomBehavior(new OrgChartBehavior(view));
	}
}